@@ -150,7 +150,7 @@ static int qc_yuv_toplanar(unsigned char *rgb24frame, int length, int format)
 	cr = crptr;
 	cbptr = &rgb24frame[length+n];
 	cb = cbptr;
-	if (debug&QC_DEBUGLOGIC) PDEBUG("qc_yuv_toplanar: %d (%d + 2 * %d)",length, l, n);
+	if (qcdebug&QC_DEBUGLOGIC) PDEBUG("qc_yuv_toplanar: %d (%d + 2 * %d)",length, l, n);
 
 	/* separate Y , U and V */
 	for (i=0; i<length; i+=2) {
@@ -183,16 +183,15 @@ static int qc_yuv_toplanar(unsigned char *rgb24frame, int length, int format)
  */
 int qc_yuv_rgb2yuv(unsigned char *rgb24frame, int length, int format)
 {
-	int i,l;
+	int i;
+	int l;			/* Data length */
 	short Y;
 	short U;
 	short V;
 	unsigned char *ptr;
 	unsigned short w;
-	//unsigned char *rgb24frame = frame->data;
-	//int length = frame->scanlength;
 
-	//if (debug&QC_DEBUGLOGIC)PDEBUG("qc_yuv_rgb2yuv(frame=%p,length=%i,palette=%s)",rgb24frame,length,qc_fmt_getname(format));
+	//if (qcdebug&QC_DEBUGLOGIC)PDEBUG("qc_yuv_rgb2yuv(frame=%p,length=%i,palette=%s)",rgb24frame,length,qc_fmt_getname(format));
 	ptr = rgb24frame;
 	l = 0;
 
@@ -241,8 +240,8 @@ int qc_yuv_rgb2yuv(unsigned char *rgb24frame, int length, int format)
 			/* take top five bits and pack while switch RGB with BGR */
 			/* FIXME: do we need to take into account of non-x86 byte ordering? */
 			w = ((unsigned short)
-				((unsigned short) (rgb24frame[i + 2] & 0xf8) << (11 - 3)) |
-				((unsigned short) (rgb24frame[i + 1] & 0xfc) << (5 - 2)) |
+				((unsigned short) (rgb24frame[i + 2] & 0xF8) << (11 - 3)) |
+				((unsigned short) (rgb24frame[i + 1] & 0xFC) << (5 - 2)) |
 				((unsigned short) (rgb24frame[i + 0] >> 3)));
 			*ptr++ = (unsigned char) (w & 0xFF);
 			*ptr++ = (unsigned char) (w >> 8);
@@ -271,7 +270,7 @@ int qc_yuv_rgb2yuv(unsigned char *rgb24frame, int length, int format)
 	if (format >= VIDEO_PALETTE_PLANAR) {
 		length = qc_yuv_toplanar(rgb24frame, l, format);
 	} else {
-		length = 0;	//FIXME:get actual length
+		length = l;
 	}
 	return length;
 }
@@ -358,6 +357,28 @@ static unsigned char qc_imag_rgb24_midvalue(unsigned char *rgb, int rgb_line,
 }
 /* }}} */
 #endif
+/* {{{ [fold] qc_imag_userlut(unsigned char (*userlut)[QC_LUT_SIZE], unsigned char (*lut)[QC_LUT_SIZE]) */
+/* Apply user-defined lookup-table to the given preinitialized lut */
+static inline void qc_imag_userlut(unsigned char (*userlut)[QC_LUT_SIZE], unsigned char (*lut)[QC_LUT_SIZE])
+{
+	unsigned int i,p;
+	for (i=0; i<256; i++) {
+		p = (*lut)[QC_LUT_RED + i];
+		p = (*userlut)[QC_LUT_RED + p];
+		(*lut)[QC_LUT_RED + i] = p;
+	}
+	for (i=0; i<256; i++) {
+		p = (*lut)[QC_LUT_GREEN + i];
+		p = (*userlut)[QC_LUT_GREEN + p];
+		(*lut)[QC_LUT_GREEN + i] = p;
+	}
+	for (i=0; i<256; i++) {
+		p = (*lut)[QC_LUT_BLUE + i];
+		p = (*userlut)[QC_LUT_BLUE + p];
+		(*lut)[QC_LUT_BLUE + i] = p;
+	}
+}
+/* }}} */
 /* {{{ [fold] qc_imag_bayer_equalize(char *bay, int bay_line, int columns, int rows, char (*lut)[]) */
 /* Compute and fill a lookup table which will equalize the image. 
  * bay = points to the bayer image data (upper left pixel is green)
@@ -366,7 +387,7 @@ static unsigned char qc_imag_rgb24_midvalue(unsigned char *rgb, int rgb_line,
  * lut = lookup table to be filled, 3*256 char array
  */
 static void qc_imag_bayer_equalize(unsigned char *bay, int bay_line, 
-	unsigned int columns, unsigned int rows, unsigned char (*lut)[LUT_SIZE])
+	unsigned int columns, unsigned int rows, unsigned char (*lut)[QC_LUT_SIZE])
 {
 	static const unsigned int stepsize = 4;		/* Larger = faster and less accurate */
 	unsigned short red_cnt[256], green_cnt[256], blue_cnt[256];	/* FIXME: how much we can use stack? */
@@ -405,9 +426,9 @@ static void qc_imag_bayer_equalize(unsigned char *bay, int bay_line,
 	green_sum = 0;
 	blue_sum  = 0;
 	for (i=0; i<256; i++) {
-		(*lut)[LUT_RED   + i] = 255 * red_sum   / red_tot;
-		(*lut)[LUT_GREEN + i] = 255 * green_sum / green_tot;
-		(*lut)[LUT_BLUE  + i] = 255 * blue_sum  / blue_tot;
+		(*lut)[QC_LUT_RED   + i] = 255 * red_sum   / red_tot;
+		(*lut)[QC_LUT_GREEN + i] = 255 * green_sum / green_tot;
+		(*lut)[QC_LUT_BLUE  + i] = 255 * blue_sum  / blue_tot;
 		red_sum   += red_cnt[i];
 		green_sum += green_cnt[i];
 		blue_sum  += blue_cnt[i];
@@ -422,7 +443,7 @@ static void qc_imag_bayer_equalize(unsigned char *bay, int bay_line,
  * lut = lookup table to be used, 3*256 char array
  */
 static void qc_imag_bayer_lut(unsigned char *bay, int bay_line, 
-	unsigned int columns, unsigned int rows, unsigned char (*lut)[LUT_SIZE])
+	unsigned int columns, unsigned int rows, unsigned char (*lut)[QC_LUT_SIZE])
 {
 	unsigned char *cur_bay;
 	unsigned int total_columns;
@@ -433,16 +454,16 @@ static void qc_imag_bayer_lut(unsigned char *bay, int bay_line,
 		columns = total_columns;
 		cur_bay = bay;
 		do {
-			cur_bay[0] = (*lut)[LUT_GREEN + cur_bay[0]];
-			cur_bay[1] = (*lut)[LUT_RED   + cur_bay[1]];
+			cur_bay[0] = (*lut)[QC_LUT_GREEN + cur_bay[0]];
+			cur_bay[1] = (*lut)[QC_LUT_RED   + cur_bay[1]];
 			cur_bay += 2;
 		} while (--columns);
 		bay += bay_line;
 		columns = total_columns;
 		cur_bay = bay;
 		do {
-			cur_bay[0] = (*lut)[LUT_BLUE  + cur_bay[0]];
-			cur_bay[1] = (*lut)[LUT_GREEN + cur_bay[1]];
+			cur_bay[0] = (*lut)[QC_LUT_BLUE  + cur_bay[0]];
+			cur_bay[1] = (*lut)[QC_LUT_GREEN + cur_bay[1]];
 			cur_bay += 2;
 		} while (--columns);
 		bay += bay_line;
@@ -838,6 +859,241 @@ static inline void qc_imag_bay2rgb_cottnoip(unsigned char *bay, int bay_line,
 	qc_imag_writergb(cur_rgb+rgb_line+bpp, bpp, cur_bay[1], cur_bay[bay_line+1], cur_bay[bay_line]);
 }
 /* }}} */
+/* {{{ [fold] qc_imag_bay2rgb_gptm_fast(unsigned char *bay, int bay_line, unsigned char *rgb, int rgb_line, int columns, int rows, int bpp) */
+/* Convert Bayer image to RGB image using Generalized Pei-Tam method
+ * Uses fixed weights */
+/* Execution time: 3795517 clock cycles */
+static inline void qc_imag_bay2rgb_gptm_fast(unsigned char *bay, int bay_line,
+		   unsigned char *rgb, int rgb_line,
+		   int columns, int rows, int bpp)
+{
+	int r,g,b,w;
+	unsigned char *cur_bay, *cur_rgb;
+	int bay_line2, bay_line3, rgb_line2;
+	int total_columns;
+
+	/* Process 2 lines and rows per each iteration, but process the first and last two columns and rows separately */
+	total_columns = (columns>>1) - 2;
+	rows = (rows>>1) - 2;
+	bay_line2 = 2*bay_line;
+	bay_line3 = 3*bay_line;
+	rgb_line2 = 2*rgb_line;
+
+	/* Process first two pixel rows here */
+	cur_bay = bay;
+	cur_rgb = rgb;
+	columns = total_columns + 2;
+	do {
+		qc_imag_writergb(cur_rgb+0,            bpp, cur_bay[1], cur_bay[0],          cur_bay[bay_line]);
+		qc_imag_writergb(cur_rgb+bpp,          bpp, cur_bay[1], cur_bay[0],          cur_bay[bay_line]);
+		qc_imag_writergb(cur_rgb+rgb_line,     bpp, cur_bay[1], cur_bay[bay_line+1], cur_bay[bay_line]);
+		qc_imag_writergb(cur_rgb+rgb_line+bpp, bpp, cur_bay[1], cur_bay[bay_line+1], cur_bay[bay_line]);
+		cur_bay += 2;
+		cur_rgb += 2*bpp;
+	} while (--columns);
+	bay += bay_line2;
+	rgb += rgb_line2;
+
+	do {
+		cur_bay = bay;
+		cur_rgb = rgb;
+		columns = total_columns;
+		
+		/* Process first 2x2 pixel block in a row here */
+		qc_imag_writergb(cur_rgb+0,            bpp, cur_bay[1], cur_bay[0],          cur_bay[bay_line]);
+		qc_imag_writergb(cur_rgb+bpp,          bpp, cur_bay[1], cur_bay[0],          cur_bay[bay_line]);
+		qc_imag_writergb(cur_rgb+rgb_line,     bpp, cur_bay[1], cur_bay[bay_line+1], cur_bay[bay_line]);
+		qc_imag_writergb(cur_rgb+rgb_line+bpp, bpp, cur_bay[1], cur_bay[bay_line+1], cur_bay[bay_line]);
+		cur_bay += 2;
+		cur_rgb += 2*bpp;
+
+		do {
+			w = 4*cur_bay[0] - (cur_bay[-bay_line-1] + cur_bay[-bay_line+1] + cur_bay[bay_line-1] + cur_bay[bay_line+1]);
+			r = (2*(cur_bay[-1] + cur_bay[1]) + w) >> 2;
+			b = (2*(cur_bay[-bay_line] + cur_bay[bay_line]) + w) >> 2;
+			qc_imag_writergb(cur_rgb+0, bpp, CLIP(r,0,255), cur_bay[0], CLIP(b,0,255));
+
+			w = 4*cur_bay[1] - (cur_bay[-bay_line2+1] + cur_bay[-1] + cur_bay[3] + cur_bay[bay_line2+1]);
+			g = (2*(cur_bay[-bay_line+1] + cur_bay[0] + cur_bay[2] + cur_bay[bay_line+1]) + w) >> 3;
+			b = (2*(cur_bay[-bay_line] + cur_bay[-bay_line+2] + cur_bay[bay_line] + cur_bay[bay_line+2]) + w) >> 3;
+			qc_imag_writergb(cur_rgb+bpp, bpp, cur_bay[1], CLIP(g,0,255), CLIP(b,0,255));
+
+			w = 4*cur_bay[bay_line] - (cur_bay[-bay_line] + cur_bay[bay_line-2] + cur_bay[bay_line+2] + cur_bay[bay_line3]);
+			r = ((cur_bay[-1] + cur_bay[1] + cur_bay[bay_line2-1] + cur_bay[bay_line2+1]) + w) >> 2;
+			g = ((cur_bay[0] + cur_bay[bay_line-1] + cur_bay[bay_line+1] + cur_bay[bay_line2]) + w) >> 2;
+			qc_imag_writergb(cur_rgb+rgb_line, bpp, CLIP(r,0,255), CLIP(g,0,255), cur_bay[bay_line]);
+
+			w = 4*cur_bay[bay_line+1] - (cur_bay[0] + cur_bay[2] + cur_bay[bay_line2] + cur_bay[bay_line2+2]);
+			r = (2*(cur_bay[1] + cur_bay[bay_line2+1]) + w) >> 2;
+			b = (2*(cur_bay[bay_line] + cur_bay[bay_line+2]) + w) >> 2;
+			qc_imag_writergb(cur_rgb+rgb_line+bpp, bpp, CLIP(r,0,255), cur_bay[bay_line+1], CLIP(b,0,255));
+
+			cur_bay += 2;
+			cur_rgb += 2*bpp;
+		} while (--columns);
+
+		/* Process last 2x2 pixel block in a row here */
+		qc_imag_writergb(cur_rgb+0,            bpp, cur_bay[1], cur_bay[0],          cur_bay[bay_line]);
+		qc_imag_writergb(cur_rgb+bpp,          bpp, cur_bay[1], cur_bay[0],          cur_bay[bay_line]);
+		qc_imag_writergb(cur_rgb+rgb_line,     bpp, cur_bay[1], cur_bay[bay_line+1], cur_bay[bay_line]);
+		qc_imag_writergb(cur_rgb+rgb_line+bpp, bpp, cur_bay[1], cur_bay[bay_line+1], cur_bay[bay_line]);
+
+		bay += bay_line2;
+		rgb += rgb_line2;
+	} while (--rows);
+
+	/* Process last two pixel rows here */
+	cur_bay = bay;
+	cur_rgb = rgb;
+	columns = total_columns + 2;
+	do {
+		qc_imag_writergb(cur_rgb+0,            bpp, cur_bay[1], cur_bay[0],          cur_bay[bay_line]);
+		qc_imag_writergb(cur_rgb+bpp,          bpp, cur_bay[1], cur_bay[0],          cur_bay[bay_line]);
+		qc_imag_writergb(cur_rgb+rgb_line,     bpp, cur_bay[1], cur_bay[bay_line+1], cur_bay[bay_line]);
+		qc_imag_writergb(cur_rgb+rgb_line+bpp, bpp, cur_bay[1], cur_bay[bay_line+1], cur_bay[bay_line]);
+		cur_bay += 2;
+		cur_rgb += 2*bpp;
+	} while (--columns);
+}
+/* }}} */
+/* {{{ [fold] qc_imag_bay2rgb_gptm(unsigned char *bay, int bay_line, unsigned char *rgb, int rgb_line, int columns, int rows, int bpp, int sharpness) */
+/* Convert Bayer image to RGB image using Generalized Pei-Tam method (See:
+ * "Effective Color Interpolation in CCD Color Filter Arrays Using Signal Correlation"
+ * IEEE Transactions on Circuits and Systems for Video Technology, vol. 13, no. 6, June 2003.
+ * Note that this is much improved version of the algorithm described in the paper)
+ * bay = points to the bayer image data (upper left pixel is green)
+ * bay_line = bytes between the beginnings of two consecutive rows
+ * rgb = points to the rgb image data that is written
+ * rgb_line = bytes between the beginnings of two consecutive rows
+ * columns, rows = bayer image size (both must be even)
+ * bpp = number of bytes in each pixel in the RGB image (should be 3 or 4)
+ * sharpness = how sharp the image should be, between 0..65535 inclusive.
+ *             23170 gives in theory image that corresponds to the original
+ *             best, but human eye likes slightly sharper picture... 32768 is a good bet.
+ *             When sharpness = 0, this routine is same as bilinear interpolation.
+ */
+/* Execution time: 4344042 clock cycles for CIF image (Pentium II) */
+static inline void qc_imag_bay2rgb_gptm(unsigned char *bay, int bay_line,
+		   unsigned char *rgb, int rgb_line,
+		   int columns, int rows, int bpp, unsigned int sharpness)
+{
+
+	/* 0.8 fixed point weights, should be between 0-256. Larger value = sharper, zero corresponds to bilinear interpolation. */
+	/* Best PSNR with sharpness = 23170 */
+	static const int wrg0 = 144;		/* Weight for Red on Green */
+	static const int wbg0 = 160;
+	static const int wgr0 = 120;
+	static const int wbr0 = 192;
+	static const int wgb0 = 120;
+	static const int wrb0 = 168;
+
+	int wrg;
+	int wbg;
+	int wgr;
+	int wbr;
+	int wgb;
+	int wrb;
+
+	unsigned int wu;
+	int r,g,b,w;
+	unsigned char *cur_bay, *cur_rgb;
+	int bay_line2, bay_line3, rgb_line2;
+	int total_columns;
+
+	/* Compute weights */
+	wu = (sharpness * sharpness) >> 16;
+ 	wu = (wu * wu) >> 16;
+	wrg = (wrg0 * wu) >> 10;
+	wbg = (wbg0 * wu) >> 10;
+	wgr = (wgr0 * wu) >> 10;
+	wbr = (wbr0 * wu) >> 10;
+	wgb = (wgb0 * wu) >> 10;
+	wrb = (wrb0 * wu) >> 10;
+
+	/* Process 2 lines and rows per each iteration, but process the first and last two columns and rows separately */
+	total_columns = (columns>>1) - 2;
+	rows = (rows>>1) - 2;
+	bay_line2 = 2*bay_line;
+	bay_line3 = 3*bay_line;
+	rgb_line2 = 2*rgb_line;
+
+	/* Process first two pixel rows here */
+	cur_bay = bay;
+	cur_rgb = rgb;
+	columns = total_columns + 2;
+	do {
+		qc_imag_writergb(cur_rgb+0,            bpp, cur_bay[1], cur_bay[0],          cur_bay[bay_line]);
+		qc_imag_writergb(cur_rgb+bpp,          bpp, cur_bay[1], cur_bay[0],          cur_bay[bay_line]);
+		qc_imag_writergb(cur_rgb+rgb_line,     bpp, cur_bay[1], cur_bay[bay_line+1], cur_bay[bay_line]);
+		qc_imag_writergb(cur_rgb+rgb_line+bpp, bpp, cur_bay[1], cur_bay[bay_line+1], cur_bay[bay_line]);
+		cur_bay += 2;
+		cur_rgb += 2*bpp;
+	} while (--columns);
+	bay += bay_line2;
+	rgb += rgb_line2;
+
+	do {
+		cur_bay = bay;
+		cur_rgb = rgb;
+		columns = total_columns;
+		
+		/* Process first 2x2 pixel block in a row here */
+		qc_imag_writergb(cur_rgb+0,            bpp, cur_bay[1], cur_bay[0],          cur_bay[bay_line]);
+		qc_imag_writergb(cur_rgb+bpp,          bpp, cur_bay[1], cur_bay[0],          cur_bay[bay_line]);
+		qc_imag_writergb(cur_rgb+rgb_line,     bpp, cur_bay[1], cur_bay[bay_line+1], cur_bay[bay_line]);
+		qc_imag_writergb(cur_rgb+rgb_line+bpp, bpp, cur_bay[1], cur_bay[bay_line+1], cur_bay[bay_line]);
+		cur_bay += 2;
+		cur_rgb += 2*bpp;
+
+		do {
+			w = 4*cur_bay[0] - (cur_bay[-bay_line-1] + cur_bay[-bay_line+1] + cur_bay[bay_line-1] + cur_bay[bay_line+1]);
+			r = (512*(cur_bay[-1] + cur_bay[1]) + w*wrg) >> 10;
+			b = (512*(cur_bay[-bay_line] + cur_bay[bay_line]) + w*wbg) >> 10;
+			qc_imag_writergb(cur_rgb+0, bpp, CLIP(r,0,255), cur_bay[0], CLIP(b,0,255));
+
+			w = 4*cur_bay[1] - (cur_bay[-bay_line2+1] + cur_bay[-1] + cur_bay[3] + cur_bay[bay_line2+1]);
+			g = (256*(cur_bay[-bay_line+1] + cur_bay[0] + cur_bay[2] + cur_bay[bay_line+1]) + w*wgr) >> 10;
+			b = (256*(cur_bay[-bay_line] + cur_bay[-bay_line+2] + cur_bay[bay_line] + cur_bay[bay_line+2]) + w*wbr) >> 10;
+			qc_imag_writergb(cur_rgb+bpp, bpp, cur_bay[1], CLIP(g,0,255), CLIP(b,0,255));
+
+			w = 4*cur_bay[bay_line] - (cur_bay[-bay_line] + cur_bay[bay_line-2] + cur_bay[bay_line+2] + cur_bay[bay_line3]);
+			r = (256*(cur_bay[-1] + cur_bay[1] + cur_bay[bay_line2-1] + cur_bay[bay_line2+1]) + w*wrb) >> 10;
+			g = (256*(cur_bay[0] + cur_bay[bay_line-1] + cur_bay[bay_line+1] + cur_bay[bay_line2]) + w*wgb) >> 10;
+			qc_imag_writergb(cur_rgb+rgb_line, bpp, CLIP(r,0,255), CLIP(g,0,255), cur_bay[bay_line]);
+
+			w = 4*cur_bay[bay_line+1] - (cur_bay[0] + cur_bay[2] + cur_bay[bay_line2] + cur_bay[bay_line2+2]);
+			r = (512*(cur_bay[1] + cur_bay[bay_line2+1]) + w*wrg) >> 10;
+			b = (512*(cur_bay[bay_line] + cur_bay[bay_line+2]) + w*wbg) >> 10;
+			qc_imag_writergb(cur_rgb+rgb_line+bpp, bpp, CLIP(r,0,255), cur_bay[bay_line+1], CLIP(b,0,255));
+
+			cur_bay += 2;
+			cur_rgb += 2*bpp;
+		} while (--columns);
+
+		/* Process last 2x2 pixel block in a row here */
+		qc_imag_writergb(cur_rgb+0,            bpp, cur_bay[1], cur_bay[0],          cur_bay[bay_line]);
+		qc_imag_writergb(cur_rgb+bpp,          bpp, cur_bay[1], cur_bay[0],          cur_bay[bay_line]);
+		qc_imag_writergb(cur_rgb+rgb_line,     bpp, cur_bay[1], cur_bay[bay_line+1], cur_bay[bay_line]);
+		qc_imag_writergb(cur_rgb+rgb_line+bpp, bpp, cur_bay[1], cur_bay[bay_line+1], cur_bay[bay_line]);
+
+		bay += bay_line2;
+		rgb += rgb_line2;
+	} while (--rows);
+
+	/* Process last two pixel rows here */
+	cur_bay = bay;
+	cur_rgb = rgb;
+	columns = total_columns + 2;
+	do {
+		qc_imag_writergb(cur_rgb+0,            bpp, cur_bay[1], cur_bay[0],          cur_bay[bay_line]);
+		qc_imag_writergb(cur_rgb+bpp,          bpp, cur_bay[1], cur_bay[0],          cur_bay[bay_line]);
+		qc_imag_writergb(cur_rgb+rgb_line,     bpp, cur_bay[1], cur_bay[bay_line+1], cur_bay[bay_line]);
+		qc_imag_writergb(cur_rgb+rgb_line+bpp, bpp, cur_bay[1], cur_bay[bay_line+1], cur_bay[bay_line]);
+		cur_bay += 2;
+		cur_rgb += 2*bpp;
+	} while (--columns);
+}
+/* }}} */
 /* {{{ [fold] qc_imag_rgbbgr(unsigned char *dst, int pixels, int bpp) */
 /* Convert RGB image to BGR or vice versa with the given number of pixels and
  * bytes per pixel
@@ -1032,7 +1288,7 @@ int qc_fmt_getdepth(int palette)
 int qc_fmt_init(struct quickcam *qc)
 {
 	int r = 0;
-	if (debug&QC_DEBUGLOGIC) PDEBUG("qc_fmt_init(qc=%p/compr=%i)",qc,qc->settings.compress);
+	if (qcdebug&QC_DEBUGLOGIC) PDEBUG("qc_fmt_init(qc=%p/compr=%i)",qc,qc->settings.compress);
 #if COMPRESS
 	if (qc->settings.compress) {
 		qc->fmt_data.compress = TRUE;
@@ -1048,12 +1304,15 @@ int qc_fmt_init(struct quickcam *qc)
 /* {{{ [fold] qc_fmt_exit(struct quickcam *qc) */
 void qc_fmt_exit(struct quickcam *qc)
 {
-	if (debug&QC_DEBUGLOGIC) PDEBUG("qc_fmt_exit(qc=%p)",qc);
+	if (qcdebug&QC_DEBUGLOGIC) PDEBUG("qc_fmt_exit(qc=%p)",qc);
 #if COMPRESS
-	if (debug&QC_DEBUGLOGIC) PDEBUG("qc_fmt_exit/compress=%i",qc->fmt_data.compress);
+	if (qcdebug&QC_DEBUGLOGIC) PDEBUG("qc_fmt_exit/compress=%i",qc->fmt_data.compress);
 	if (qc->fmt_data.compress) qc_mjpeg_exit(&qc->fmt_data.mjpeg_data);
+	POISON(qc->fmt_data.compress);
+	POISON(qc->fmt_data.mjpeg_data);
 #endif
-	IDEBUG_EXIT(qc->fmt_data);
+	POISON(qc->fmt_data.lut);
+	IDEBUG_EXIT_NOPOISON(qc->fmt_data);
 }
 /* }}} */
 /* {{{ [fold] qc_fmt_convert(struct quickcam *qc, unsigned char *src, unsigned int src_len, unsigned char *dst, unsigned int dst_len, int *midvalue) */
@@ -1070,21 +1329,29 @@ int qc_fmt_convert(struct quickcam *qc, unsigned char *src, unsigned int src_len
 	int length;		/* Converted image data length in bytes */
 	int r = 0;
 
-	if (debug&QC_DEBUGLOGIC) PDEBUG("qc_imag_convert(qc=%p,src=%p,src_len=%i,dst=%p,dst_len=%i)",qc,src,src_len,dst,dst_len);
+	if (qcdebug&QC_DEBUGLOGIC) PDEBUG("qc_imag_convert(qc=%p,src=%p,src_len=%i,dst=%p,dst_len=%i)",qc,src,src_len,dst,dst_len);
 	if (PARANOID && (qc==NULL || src==NULL || dst==NULL)) { PRINTK(KERN_CRIT,"NULL"); return -EINVAL; }
 	if (PARANOID && midvalue==NULL) { PRINTK(KERN_CRIT,"NULL"); return -EINVAL; }
 	IDEBUG_TEST(qc->fmt_data);
 	*midvalue = -1;
 
-	if (debug&QC_DEBUGLOGIC) PDEBUG("sensor_width=%i sensor_height=%i vwin.width=%i vwin.height=%i",
+	if (qcdebug&QC_DEBUGLOGIC) PDEBUG("sensor_width=%i sensor_height=%i vwin.width=%i vwin.height=%i",
 			qc->sensor_data.width,qc->sensor_data.height,qc->vwin.width,qc->vwin.height);
 
 	if (qc->sensor_data.width < qc->vwin.width || qc->sensor_data.height < qc->vwin.height) {
-		if (debug&QC_DEBUGERRORS) PDEBUG("sensor window is smaller than requested");
+		if (qcdebug&QC_DEBUGERRORS) PDEBUG("sensor window is smaller than requested");
 		r = -ENOMSG;
 		goto fail;
 	}
 
+#if DUMPDATA
+	if (midvalue) *midvalue = -1;
+	memset(dst, 0, dst_len);
+	memcpy(dst, src, src_len>dst_len ? dst_len : src_len);
+	length = (int)qc->vwin.width * (int)qc->vwin.height * 3;
+	return length;
+#endif
+
 #if COMPRESS
 	if (!qc->sensor_data.compress) {
 #endif
@@ -1094,24 +1361,33 @@ int qc_fmt_convert(struct quickcam *qc, unsigned char *src, unsigned int src_len
 
 		/* Check if src buffer contains enough data */
 		if (src_len < qc->sensor_data.width * qc->sensor_data.height) {
-			if (debug&QC_DEBUGERRORS) PDEBUG("too little data by %i (expected %i)", qc->sensor_data.width*qc->sensor_data.height - src_len, qc->sensor_data.width * qc->sensor_data.height);
-			r = -EMSGSIZE;
+			if (qcdebug&QC_DEBUGERRORS) PDEBUG("too little data by %i (expected %i)", qc->sensor_data.width*qc->sensor_data.height - src_len, qc->sensor_data.width * qc->sensor_data.height);
+			r = -EBADE;
 			goto fail;
 		}
-
 		/* calculate view window origin inside the whole frame */
 		originy = ((signed int)qc->sensor_data.height - (signed int)qc->vwin.height) / 2;
 		originx = ((signed int)qc->sensor_data.width - (signed int)qc->vwin.width) / 2;
 		originx &= ~1;		/* Set upper-left corner to a even coordinate */
 		originy &= ~1;		/* so that the first bayer pixel is green */
 		bayerwin = src + originy * qc->sensor_data.width + originx;
-		if (debug&QC_DEBUGLOGIC) PDEBUG("originy=%i  originx=%i", originy,originx);
+		if (qcdebug&QC_DEBUGLOGIC) PDEBUG("originy=%i  originx=%i", originy,originx);
 
 		if (qc->settings.adaptive && midvalue!=NULL)
 			*midvalue = qc_imag_bayer_midvalue(bayerwin, qc->sensor_data.width, qc->vwin.width, qc->vwin.height);
-
-		if (qc->settings.equalize) {
-			qc_imag_bayer_equalize(bayerwin, qc->sensor_data.width, qc->vwin.width, qc->vwin.height, &qc->fmt_data.lut);
+		if (qc->settings.equalize || qc->settings.userlut) {
+			if (qc->settings.equalize) {
+				qc_imag_bayer_equalize(bayerwin, qc->sensor_data.width, qc->vwin.width, qc->vwin.height, &qc->fmt_data.lut);
+			} else {
+				/* Initialize LUT */
+				int i;
+				for (i=0; i<256; i++) qc->fmt_data.lut[QC_LUT_RED+i]   = i;
+				for (i=0; i<256; i++) qc->fmt_data.lut[QC_LUT_GREEN+i] = i;
+				for (i=0; i<256; i++) qc->fmt_data.lut[QC_LUT_BLUE+i]  = i;
+			}
+			if (qc->settings.userlut) {
+				qc_imag_userlut(&qc->fmt_data.userlut, &qc->fmt_data.lut);
+			}
 			/* Could do here other effects to the lookup table */
 			qc_imag_bayer_lut(bayerwin, qc->sensor_data.width, qc->vwin.width, qc->vwin.height, &qc->fmt_data.lut);
 		}
@@ -1153,12 +1429,22 @@ int qc_fmt_convert(struct quickcam *qc, unsigned char *src, unsigned int src_len
 						dst, 3*qc->vwin.width, 
 						qc->vwin.width, qc->vwin.height, 3);
 				break;
-			default:
 			case 3:
 				qc_imag_bay2rgb_ip(bayerwin, qc->sensor_data.width,
 						dst, 3*qc->vwin.width, 
 						qc->vwin.width, qc->vwin.height, 3);
 				break;
+			default:
+			case 4:
+				qc_imag_bay2rgb_gptm_fast(bayerwin, qc->sensor_data.width,
+						dst, 3*qc->vwin.width, 
+						qc->vwin.width, qc->vwin.height, 3);
+				break;
+			case 5:
+				qc_imag_bay2rgb_gptm(bayerwin, qc->sensor_data.width,
+						dst, 3*qc->vwin.width, 
+						qc->vwin.width, qc->vwin.height, 3, qc->vpic.whiteness);
+				break;
 			}
 			goto rgb24;
 		}
@@ -1213,33 +1499,33 @@ int qc_fmt_convert(struct quickcam *qc, unsigned char *src, unsigned int src_len
 rgb24:	/* We have now RGB (24 bpp) data in dst. If some other format is desired, */
 	/* convert the RGB image to it (e.g. YUV) */
 
-#ifndef NDEBUG
+#ifdef DEBUG
 #if 1
 {
-/* Draw blue rectangle around image (useful for debugging boundaries) */
-static const int R = 0;
+/* Draw red rectangle around image (useful for debugging boundaries) */
+static const int R = 255;
 static const int G = 0;
-static const int B = 255;
+static const int B = 0;
  int ty,tx;
  for (tx=0; tx<qc->vwin.width; tx++) {
   ty=0;
-  dst[ty*qc->vwin.width*3+tx*3] = R;
+  dst[ty*qc->vwin.width*3+tx*3] = B;
   dst[ty*qc->vwin.width*3+tx*3+1] = G;
-  dst[ty*qc->vwin.width*3+tx*3+2] = B;
+  dst[ty*qc->vwin.width*3+tx*3+2] = R;
   ty=qc->vwin.height-1;
-  dst[ty*qc->vwin.width*3+tx*3] = R;
+  dst[ty*qc->vwin.width*3+tx*3] = B;
   dst[ty*qc->vwin.width*3+tx*3+1] = G;
-  dst[ty*qc->vwin.width*3+tx*3+2] = B;
+  dst[ty*qc->vwin.width*3+tx*3+2] = R;
  }
  for (ty=0; ty<qc->vwin.height; ty++) {
   tx=0;
-  dst[ty*qc->vwin.width*3+tx*3] = R;
+  dst[ty*qc->vwin.width*3+tx*3] = B;
   dst[ty*qc->vwin.width*3+tx*3+1] = G;
-  dst[ty*qc->vwin.width*3+tx*3+2] = B;
+  dst[ty*qc->vwin.width*3+tx*3+2] = R;
   tx=qc->vwin.width-1;
-  dst[ty*qc->vwin.width*3+tx*3] = R;
+  dst[ty*qc->vwin.width*3+tx*3] = B;
   dst[ty*qc->vwin.width*3+tx*3+1] = G;
-  dst[ty*qc->vwin.width*3+tx*3+2] = B;
+  dst[ty*qc->vwin.width*3+tx*3+2] = R;
  }
 }
 #endif
@@ -1253,7 +1539,7 @@ static const int B = 255;
 	}
 	return length;
 
-fail:	if (debug&(QC_DEBUGERRORS|QC_DEBUGLOGIC)) PDEBUG("failed qc_imag_convert()=%i",r);
+fail:	if (qcdebug&(QC_DEBUGERRORS|QC_DEBUGLOGIC)) PDEBUG("failed qc_imag_convert()=%i",r);
 	return r;
 }
 /* }}} */
