@@ -29,6 +29,10 @@
 #include <linux/quickcam.h>
 #endif
 
+#ifndef QCEGA_MODE
+#define QCEGA_MODE 0		/* If the driver doesn't work for you, try changing this to "1" */
+#endif
+
 /* LSB bit of I2C address signifies write (0) or read (1) */
 
 /* I2C Address */
@@ -104,11 +108,77 @@
 #define STV_SETW_CHECK(reg,val)	if ((r = qc_stv_setw(qc,(reg),(val)))<0) goto fail
 #define IS_850(qc)		(GET_PRODUCTID(qc)==0x850)	/* Is it QuickCam Web/Legocam? */
 
+#if QCEGA_MODE
+#warning "Using old compatible code (QCEGA_MODE=1)"
+#warning "If this works but otherwise it doesn't work, let me know!"
+static int mode = 0;
+#define VV6410_CONTROL 		0x10	// Setup0
+#define VV6410_GAIN  		0x24
+/* {{{ [fold] vv6410_set_window() */
+static int vv6410_set_window(struct quickcam *qc, int x, int y,int width, int height)
+{
+	int r = 0;
+
+	// x offset
+        x = MAX(1,x);
+	I2C_SET_CHECK(0x57,x >> 8);
+	I2C_SET_CHECK(0x58,x & 0xff);
+
+	// y offset
+        y = MAX(1,y);
+	I2C_SET_CHECK(0x59,y >> 8);
+	I2C_SET_CHECK(0x5a,y & 0xff);
+
+        // Set the real
+        if (qc->sensor_data.subsample) {
+            qc->sensor_data.width=180;
+            qc->sensor_data.height=148;
+        } else {
+            qc->sensor_data.width=356;
+            qc->sensor_data.height=292;
+        }
+
+	// line length
+        if (qc->sensor_data.subsample) {
+            if (IS_850(qc))
+                width=250;
+            else
+                width=360; /* 180 * 2 (CLK-DIV is 2) */
+        }
+        else {
+          if (IS_850(qc))
+                width=416;
+          else
+                width=712; /* 356 * 2 */
+	}
+
+	I2C_SET_CHECK(0x52, width >> 8);
+	I2C_SET_CHECK(0x53, width & 0xff);
+
+	// field length (num lines)
+        if (qc->sensor_data.subsample)
+          height=160; /* nearest of 148 = 10 * 16 */
+        else
+          height=320; // 304; /* nearest of 292 = 19 * 16 */ 
+
+	I2C_SET_CHECK(0x61,height >> 8);
+	I2C_SET_CHECK(0x62,height & 0xff);
+
+        // usb_quickcam_i2c_add(&i2cbuff,0x25,0x02);
+	if ((r = qc_i2c_wait(qc))<0) goto fail;
+
+        return 0;
+fail:
+	return -ENAMETOOLONG;	//some silly code just for testing
+}
+/* }}} */
+#endif
+
 /* {{{ [fold] vv6410_set_size: Set window size */
 static int vv6410_set_size(struct quickcam *qc, unsigned int width, unsigned int height)
 {
 	struct qc_sensor_data *sd = &qc->sensor_data;
-	if (debug&QC_DEBUGLOGIC) PDEBUG("vv6410_set_size(quickcam=%p,width=%i,height=%i)",qc,width,height);
+	if (qcdebug&QC_DEBUGLOGIC) PDEBUG("vv6410_set_size(qc=%p,width=%i,height=%i)",qc,width,height);
 	/* VV6410 appears to always give fixed 356*292 pixels */
 	sd->width = sd->maxwidth;
 	sd->height = sd->maxheight;
@@ -121,7 +191,7 @@ static int vv6410_start(struct quickcam *qc)
 	struct qc_sensor_data *sd = &qc->sensor_data;
 	int r;
 
-	if (debug&QC_DEBUGLOGIC) PDEBUG("vv6410_start(quickcam=%p)",qc);
+	if (qcdebug&QC_DEBUGLOGIC) PDEBUG("vv6410_start(qc=%p)",qc);
 	if (PARANOID && !qc)  { PDEBUG("qc==NULL"); return -EINVAL; }
 	I2C_SET_CHECK(VV6410_SETUP0, sd->subsample ? (BIT(7)|BIT(6)) : 0x00);
 	if (IS_850(qc)) qc_stv_set(qc, 0x1445, 1);		/* Turn on LED */
@@ -132,12 +202,12 @@ fail:	return r;
 /* {{{ [fold] vv6410_stop: Stop grabbing */
 static int vv6410_stop(struct quickcam *qc)
 {
-	static const int low_power_mode = 1;
+	static const int low_power_mode = 0; //1;
 	static const int sleep_mode     = 1;
 	struct qc_sensor_data *sd = &qc->sensor_data;
 	unsigned char cmd;
 	int r;
-	if (debug&QC_DEBUGLOGIC) PDEBUG("vv6410_stop(quickcam=%p)",qc);
+	if (qcdebug&QC_DEBUGLOGIC) PDEBUG("vv6410_stop(qc=%p)",qc);
 	if (IS_850(qc)) qc_stv_set(qc, 0x1445, 0);		/* Turn off LED */
 	cmd = (sleep_mode << 1) | low_power_mode;
 	if (sd->subsample) cmd |= BIT(7)|BIT(6);		/* sub-sampled QCIF mode */
@@ -154,7 +224,7 @@ struct stv_init {
 };
 #endif
 /* {{{ [fold] vv6410_init: Initialise parameters for vv6410 sensor. */
-/* Just try to send the same commands as Windoze Quickcam soft */
+/* Just try to send the same commands as Windoze QuickCam soft */
 static int vv6410_init(struct quickcam *qc)
 {
 	struct qc_sensor_data *sd = &qc->sensor_data;
@@ -244,7 +314,7 @@ static int vv6410_init(struct quickcam *qc)
 	unsigned int y = 1;
 	int i,j;
 
-	if (debug&QC_DEBUGLOGIC) PDEBUG("vv6410_init(quickcam=%p)",qc);
+	if (qcdebug&QC_DEBUGLOGIC) PDEBUG("vv6410_init(qc=%p)",qc);
 	if (PARANOID && !qc) { PDEBUG("qc==NULL"); return -EINVAL; }
 
 	sd->width  = 320;	/* Default to compressed mode */
@@ -311,6 +381,151 @@ static int vv6410_init(struct quickcam *qc)
 		sd->maxheight = 292;
 	}
 
+
+#if QCEGA_MODE
+{
+int line_length = mode?250:416;//415;
+ 
+        if (mode) {
+           sd->subsample=1; // quater.
+           sd->width      = 180;
+           sd->height     = 148;
+        } else {
+           sd->subsample=0;
+           sd->width      = 356;
+           sd->height     = 292;
+        }
+
+        STV_SET_CHECK(STV_REG23, 5); // was 5.
+
+
+	if (!IS_850(qc)) {
+	     /* logitech quickcam web has 0x850 as idProduct */
+	     STV_SET_CHECK(0x1446, 1);
+        }
+
+
+	STV_SET_CHECK(STV_SCAN_RATE, 0x00);
+
+
+	STV_SET_CHECK(0x1423, 0x04);
+
+
+	STV_SET_CHECK(STV_REG00, 0x1b); // 0x0b
+
+
+
+	I2C_SET_CHECK(VV6410_CONTROL,0x04); // reset to defaults
+	if ((r = qc_i2c_wait(qc))<0) goto fail;
+
+
+        
+	
+	// CIF or QCIF and sleep.
+	if (IS_850(qc)) {
+		I2C_SET_CHECK(VV6410_CONTROL,(mode?0xa2:0x02));
+	} else {
+	        I2C_SET_CHECK(VV6410_CONTROL,(mode?0xc2:0x02));
+	}
+
+	if ((r = qc_i2c_wait(qc))<0) goto fail;
+
+
+
+	I2C_SET_CHECK(VV6410_GAIN,0xfb);
+	if ((r = qc_i2c_wait(qc))<0) goto fail;
+
+
+
+
+	
+	STV_SET_CHECK(STV_REG04, 0x07);
+
+	
+	STV_SET_CHECK(STV_REG03, 0x45);
+
+
+        /* set window size */
+        if ((r=vv6410_set_window(qc,0,0,48,64)) < 0) {
+                PRINTK(KERN_ERR, "vv6410_set_window failed");
+                goto fail;
+        }
+
+
+	/* EXPERIMENTAL */
+        /*
+	 * line length default is 415 so it's the value we use to 
+	 * calculate values for  registers 0x20-0x21
+	 * Ref. DS Pag. 67
+         */   
+	I2C_SET_CHECK(0x20,mode? ((line_length-23)>>8):((line_length-51)>>8));
+
+	I2C_SET_CHECK(0x21,mode?((line_length-23)&0xff):((line_length-51)&0xff));
+
+	I2C_SET_CHECK(0x22,mode?0x00:0x01);
+         //usb_quickcam_i2c_add(&i2cbuff,0x23,mode?0x9e:0x3e);
+	I2C_SET_CHECK(0x23,mode?158:318&0xff);
+	I2C_SET_CHECK(0x24,0xfa);
+         // clock divisor.
+	I2C_SET_CHECK(0x25,0x01);
+
+	if ((r = qc_i2c_wait(qc))<0) goto fail;
+
+
+	/*
+	if (isaweb(dev))
+	  {
+	    //EXPERIMENTAL: dark/black pixel cancellation
+	    usb_quickcam_i2c_add(&i2cbuff,0x3e,0x01);
+	    usb_quickcam_i2c_add(&i2cbuff,0x72,0x01);
+	    if (usb_quickcam_i2c_send(dev,&i2cbuff,VV6410_ADDR) < 0) {
+      	      printk(KERN_ERR "usb_control_msg dark/black pixel failed");
+	      goto error;
+	    }
+	  }
+	 */
+	STV_SET_CHECK(STV_REG01, 0xb7);
+
+	STV_SET_CHECK(STV_REG02, 0xa7);
+
+
+
+	// setup
+	I2C_SET_CHECK(0x11,0x18); // 0x18 or Jochen 0x40
+	I2C_SET_CHECK(0x14,0x55); // was 0x55
+	I2C_SET_CHECK(0x15,0x10); // 0x10 or Jochen:0x00
+	I2C_SET_CHECK(0x16,0x81); // Pre clock dividor.
+	I2C_SET_CHECK(0x17,0x18); // they are reserved.
+	I2C_SET_CHECK(0x18,0x00);
+	I2C_SET_CHECK(0x77,0x5e);
+	I2C_SET_CHECK(0x78,0x04);// 0x04 or Jochen:0x00
+	if (IS_850(qc)) {
+	  I2C_SET_CHECK(0x79,0x11);//audio init
+	}
+	if ((r = qc_i2c_wait(qc))<0) goto fail;
+
+
+	
+	
+	STV_SETW_CHECK(STV_ISO_SIZE, IS_850(qc)?1023:600);	  // 0x380|orig:600
+
+
+
+
+	STV_SET_CHECK(STV_Y_CTRL, 1);
+
+
+	STV_SET_CHECK(STV_SCAN_RATE, mode?0x00:0x10);
+
+
+
+	if (!IS_850(qc)) {
+	    /* logitech quickam web has 0x0850 as idProduct */
+		STV_SET_CHECK(STV_X_CTRL, 0x14);
+	}
+}
+
+#else
 	STV_SET_CHECK(0x0423,0x05);			/* Unknown register, 0x04 or 0x05 */
 	STV_SET_CHECK(0x1423,0x04);			/* Unknown register, 0x04 or 0x05 */
 	STV_SET_CHECK(0x1443,0x00);			/* Scan rate */
@@ -350,6 +565,8 @@ static int vv6410_init(struct quickcam *qc)
 	}
 
 	if ((r = qc_i2c_wait(qc))<0) goto fail;
+#endif
+
 	return 0;
 /* }}} */
 #if COMPRESS
@@ -375,7 +592,7 @@ static int vv6410_set_exposure(struct quickcam *qc, unsigned int val)
 	coarse = val / linelength;
 	if (coarse>=512) coarse = 512;
 
-	if (debug&QC_DEBUGLOGIC) PDEBUG("vv6410_set_exposure %d (%i,%i)",val,coarse,fine);
+	if (qcdebug&QC_DEBUGLOGIC) PDEBUG("vv6410_set_exposure %d (%i,%i)",val,coarse,fine);
 	I2C_SET_CHECK(VV6410_FINEH,   fine >> 8);
 	I2C_SET_CHECK(VV6410_FINEL,   fine & 0xFF);
 	I2C_SET_CHECK(VV6410_COARSEH, coarse >> 8);
@@ -391,7 +608,7 @@ static int vv6410_set_gains(struct quickcam *qc, unsigned int hue, unsigned int
 	unsigned int gain;
 	int r;
 
-	if (debug&QC_DEBUGLOGIC) PDEBUG("vv6410_set_gains %d %d %d", hue, sat, val);
+	if (qcdebug&QC_DEBUGLOGIC) PDEBUG("vv6410_set_gains %d %d %d", hue, sat, val);
 	gain = val / 256;
 	gain >>= 4;
 	if (gain > maxgain) gain = maxgain;
