@@ -383,7 +383,7 @@ unsigned int pnm_readnum(void) {
 }
 /* }}} */
 /* {{{ [fold] void pnm_open(unsigned char *name, unsigned int *width, unsigned int *height) */
-void pnm_open(unsigned char *name, unsigned int *width, unsigned int *height) {
+void pnm_open(char *name, unsigned int *width, unsigned int *height) {
 	char c1,c2,c3;
 	unsigned int maxval;
 
