@@ -434,12 +434,33 @@ if [ "$KERNELOK" != "y" ]; then
 	fi
 fi
 
+HAVE_UTSRELEASE_H=0
+UTS_FILE=""
+if test -r $KERNEL_SOURCE/include/linux/version.h && fgrep -q UTS_RELEASE $KERNEL_SOURCE/include/linux/version.h; then
+	kernsrcver=`(echo "#include <linux/version.h>"; echo "kernsrcver=UTS_RELEASE") | cpp -I $KERNEL_SOURCE/include | grep "^kernsrcver=" | cut -d \" -f 2`
+	UTS_FILE=$KERNEL_SOURCE/include/linux/version.h
+elif test -r $KERNEL_SOURCE/include/linux/utsrelease.h && fgrep -q UTS_RELEASE $KERNEL_SOURCE/include/linux/utsrelease.h; then
+	kernsrcver=`(echo "#include <linux/utsrelease.h>"; echo "kernsrcver=UTS_RELEASE") | cpp -I $KERNEL_SOURCE/include | grep "^kernsrcver=" | cut -d \" -f 2`
+	HAVE_UTSRELEASE_H=1
+	UTS_FILE=$KERNEL_SOURCE/include/linux/utsrelease.h
+fi
+if test -z "$kernsrcver"; then
+    echo "Couldn't find KERNEL_UTS version"
+    exit 1
+fi
+#	export USER_OPT="$USER_OPT -DHAVE_UTSRELEASE_H=$HAVE_UTSRELEASE_H"
+
 KERNEL_VERSION="`make | grep 'Kernel version code' | tail -n 1 | awk -F : '{print $2}' | awk '{print $1}'`"
-KERNEL_UTS=`awk -F \" '/[ 	]*\#[ 	]*define[ 	]*UTS_RELEASE[ 	]*/ { print $2 }' "$KERNEL_SOURCE/include/linux/version.h"|tail -n 1`
+#KERNEL_UTS=`awk -F \" '/[ 	]*\#[ 	]*define[ 	]*UTS_RELEASE[ 	]*/ { print $2 }' "$KERNEL_SOURCE/include/linux/version.h"|tail -n 1`
+#KERNEL_UTS="`make | grep 'Kernel version' | tail -n 1 | awk -F : '{print $2}' | awk '{print $1}'`"
+KERNEL_UTS=$kernsrcver
 MODULE_NAME="`make | grep 'Driver file name' | tail -n 1 | awk -F : '{print $2}' | awk '{print $1}'`"
 INSTALL_DIR="`make | grep 'Module install directory' | tail -n 1 | awk -F : '{print $2}' | awk '{print $1}'`"
 export MODULE_NAME
-UTS_COUNT=`grep UTS_RELEASE < "$KERNEL_SOURCE/include/linux/version.h" | wc -l`
+
+#UTS_COUNT=`grep UTS_RELEASE < "$KERNEL_SOURCE/include/linux/version.h" | wc -l`
+UTS_COUNT=`grep UTS_RELEASE < "$UTS_FILE" | wc -l`
+
 if [ $? != 0 ]; then UTS_COUNT=0; fi
 if [ $UTS_COUNT -ne 1 ]; then
 	echo "[!] Multiple kernel versions specified in linux/version.h"
@@ -696,7 +717,7 @@ if [ "$VIDEODEV" != "$VIDEODEV_REAL" ]; then
 	ls -la "$VIDEODEV_REAL"
 	echo "$VIDEODEV is a symbolic link to $VIDEODEV_REAL."
 fi
-if [ ! -r $VIDEODEV_REAL -o ! -w $VIDEODEV_REAL ]; then
+if [ ! -r "$VIDEODEV_REAL" -o ! -w "$VIDEODEV_REAL" ]; then
 	echo "[!] You don't have read/write access to $VIDEODEV."
 	echo "On many distributions, you should add yourself into the"
 	echo "\"video\" group by giving command"
